# pyright: reportConstantRedefinition=none
"""The gpib commands module.

These commands are used in the following models:
DAQ6510, DMM6500, DMM7510, SMU2450, SMU2460, SMU2461, SMU2470, SMU2601B, SMU2601B_Pulse, SMU2602B,
SMU2604B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B, SMU2651A, SMU2657A, SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - gpib.address
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Gpib(BaseTSPCmd):
    """The ``gpib`` command tree.

    Properties and methods:
        - ``.address``: The ``gpib.address`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "gpib") -> None:
        super().__init__(device, cmd_syntax)

    @property
    def address(self) -> str:
        """Access the ``gpib.address`` attribute.

        Description:
            - This attribute contains the GPIB address.

        Usage:
            - Accessing this property will send the ``print(gpib.address)`` query.
            - Setting this property to a value will send the ``gpib.address = value`` command.

        TSP Syntax:
            ```
            - gpib.address = value
            - print(gpib.address)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".address"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.address)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.address`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @address.setter
    def address(self, value: Union[str, float]) -> None:
        """Access the ``gpib.address`` attribute.

        Description:
            - This attribute contains the GPIB address.

        Usage:
            - Accessing this property will send the ``print(gpib.address)`` query.
            - Setting this property to a value will send the ``gpib.address = value`` command.

        TSP Syntax:
            ```
            - gpib.address = value
            - print(gpib.address)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".address", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.address = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.address`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



# pyright: reportConstantRedefinition=none
"""The display commands module.

These commands are used in the following models:
SMU2601B_Pulse

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - display.getlastkey()
    - display.screen
    - display.sendkey()
    - display.smuX.digits
    - display.smuX.limit.func
    - display.smuX.measure.func
    - display.waitkey()
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING, Union

from ..helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedChannel,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


# pylint: disable=too-few-public-methods
class DisplayTrigger(BaseTSPCmd):
    """The ``display.trigger`` command tree.

    Constants:
        - ``.EVENT_ID``: Selects the event that causes a trigger to be asserted on the digital
          output line when the TRIG key on the front panel is pressed.
    """

    EVENT_ID = "display.trigger.EVENT_ID"
    """str: Selects the event that causes a trigger to be asserted on the digital output line when the TRIG key on the front panel is pressed."""  # noqa: E501


class DisplaySmuxItemMeasure(BaseTSPCmd):
    """The ``display.smuX.measure`` command tree.

    Properties and methods:
        - ``.func``: The ``display.smuX.measure.func`` attribute.
    """

    @property
    def func(self) -> str:
        """Access the ``display.smuX.measure.func`` attribute.

        Description:
            - This attribute specifies the type of measurement that is being displayed.

        Usage:
            - Accessing this property will send the ``print(display.smuX.measure.func)`` query.
            - Setting this property to a value will send the ``display.smuX.measure.func = value``
              command.

        TSP Syntax:
            ```
            - display.smuX.measure.func = value
            - print(display.smuX.measure.func)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".func"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.func)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.func`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @func.setter
    def func(self, value: Union[str, float]) -> None:
        """Access the ``display.smuX.measure.func`` attribute.

        Description:
            - This attribute specifies the type of measurement that is being displayed.

        Usage:
            - Accessing this property will send the ``print(display.smuX.measure.func)`` query.
            - Setting this property to a value will send the ``display.smuX.measure.func = value``
              command.

        TSP Syntax:
            ```
            - display.smuX.measure.func = value
            - print(display.smuX.measure.func)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".func", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.func = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.func`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class DisplaySmuxItemLimit(BaseTSPCmd):
    """The ``display.smuX.limit`` command tree.

    Properties and methods:
        - ``.func``: The ``display.smuX.limit.func`` attribute.
    """

    @property
    def func(self) -> str:
        """Access the ``display.smuX.limit.func`` attribute.

        Description:
            - This attribute specifies the type of limit value setting displayed for the SMU.

        Usage:
            - Accessing this property will send the ``print(display.smuX.limit.func)`` query.
            - Setting this property to a value will send the ``display.smuX.limit.func = value``
              command.

        TSP Syntax:
            ```
            - display.smuX.limit.func = value
            - print(display.smuX.limit.func)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".func"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.func)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.func`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @func.setter
    def func(self, value: Union[str, float]) -> None:
        """Access the ``display.smuX.limit.func`` attribute.

        Description:
            - This attribute specifies the type of limit value setting displayed for the SMU.

        Usage:
            - Accessing this property will send the ``print(display.smuX.limit.func)`` query.
            - Setting this property to a value will send the ``display.smuX.limit.func = value``
              command.

        TSP Syntax:
            ```
            - display.smuX.limit.func = value
            - print(display.smuX.limit.func)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".func", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.func = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.func`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class DisplaySmuxItem(ValidatedChannel, BaseTSPCmd):
    """The ``display.smuX`` command tree.

    Properties and methods:
        - ``.digits``: The ``display.smuX.digits`` attribute.
        - ``.limit``: The ``display.smuX.limit`` command tree.
        - ``.measure``: The ``display.smuX.measure`` command tree.
    """

    def __init__(self, device: Optional["TSPControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._limit = DisplaySmuxItemLimit(device, f"{self._cmd_syntax}.limit")
        self._measure = DisplaySmuxItemMeasure(device, f"{self._cmd_syntax}.measure")

    @property
    def digits(self) -> str:
        """Access the ``display.smuX.digits`` attribute.

        Description:
            - This attribute sets the front-panel display resolution of the selected measurement.

        Usage:
            - Accessing this property will send the ``print(display.smuX.digits)`` query.
            - Setting this property to a value will send the ``display.smuX.digits = value``
              command.

        TSP Syntax:
            ```
            - display.smuX.digits = value
            - print(display.smuX.digits)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".digits"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.digits)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.digits`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @digits.setter
    def digits(self, value: Union[str, float]) -> None:
        """Access the ``display.smuX.digits`` attribute.

        Description:
            - This attribute sets the front-panel display resolution of the selected measurement.

        Usage:
            - Accessing this property will send the ``print(display.smuX.digits)`` query.
            - Setting this property to a value will send the ``display.smuX.digits = value``
              command.

        TSP Syntax:
            ```
            - display.smuX.digits = value
            - print(display.smuX.digits)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".digits", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.digits = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.digits`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def limit(self) -> DisplaySmuxItemLimit:
        """Return the ``display.smuX.limit`` command tree.

        Sub-properties and sub-methods:
            - ``.func``: The ``display.smuX.limit.func`` attribute.
        """
        return self._limit

    @property
    def measure(self) -> DisplaySmuxItemMeasure:
        """Return the ``display.smuX.measure`` command tree.

        Sub-properties and sub-methods:
            - ``.func``: The ``display.smuX.measure.func`` attribute.
        """
        return self._measure


class Display(BaseTSPCmd):
    """The ``display`` command tree.

    Constants:
        - ``.ANNUNCIATOR_4_WIRE``: 4W indicator on.
        - ``.ANNUNCIATOR_ARM``: ARM indicator on.
        - ``.ANNUNCIATOR_AUTO``: AUTO indicator on.
        - ``.ANNUNCIATOR_EDIT``: EDIT indicator on.
        - ``.ANNUNCIATOR_ERROR``: ERR indicator on.
        - ``.ANNUNCIATOR_FILTER``: FILT indicator on.
        - ``.ANNUNCIATOR_LISTEN``: LSTN indicator on.
        - ``.ANNUNCIATOR_MATH``: MATH indicator on.
        - ``.ANNUNCIATOR_REAR``: REAR indicator on.
        - ``.ANNUNCIATOR_REL``: REL indicator on.
        - ``.ANNUNCIATOR_REMOTE``: REM indicator on.
        - ``.ANNUNCIATOR_SAMPLE``: SMPL indicator on.
        - ``.ANNUNCIATOR_SRQ``: SRQ indicator on.
        - ``.ANNUNCIATOR_STAR``: * (asterisk) indicator on.
        - ``.ANNUNCIATOR_TALK``: TALK indicator on.
        - ``.ANNUNCIATOR_TRIGGER``: TRIG indicator on.
        - ``.DIGITS_4_5``: Set the front-panel display resolution to 4.5 digits.
        - ``.DIGITS_5_5``: Set the front-panel display resolution to 5.5 digits.
        - ``.DIGITS_6_5``: Set the front-panel display resolution to 6.5 digits.
        - ``.DISABLE``: Disable the front-panel keys for numeric entry when entering a value.
        - ``.DONT_SAVE``: Do not save code to nonvolatile memory.
        - ``.ENABLE``: Enable the front-panel keys for numeric entry when entering a value.
        - ``.KEY_AUTO``: Represents the range AUTO key.
        - ``.KEY_CONFIG``: Represents the CONFIG key.
        - ``.KEY_DIGITSA``: Represents the DIGITS (display resolution) key.
          For two-channel products, this is the DIGITS key for Channel A.
        - ``.KEY_DISPLAY``: Represents the DISPLAY key.
        - ``.KEY_ENTER``: Represents the ENTER key.
        - ``.KEY_EXIT``: Represents the EXIT key.
        - ``.KEY_FILTERA``: Represents the FILTER key.
          For two-channel products, this is the FILTER key for Channel A.
        - ``.KEY_LEFT``: Represents the left CURSOR key.
        - ``.KEY_LIMITA``: Represents the LIMIT key.
          For two-channel products, this is the LIMIT key for Channel A.
        - ``.KEY_LOAD``: Represents the LOAD (load test) key.
        - ``.KEY_MEASA``: Represents the MEAS (Measure) key.
          For two-channel products, this is the MEAS key for Channel A.
        - ``.KEY_MENU``: Represents the MENU key.
        - ``.KEY_MODEA``: Represents the MODE (meter mode) key
          For two-channel products, this is the MODE key for Channel A.
        - ``.KEY_NONE``: No key was pressed.
        - ``.KEY_OUTPUTA``: Represents the OUTPUT ON/OFF key.
          For two-channel products, this is the OUTPUT ON/OFF key for Channel A.
        - ``.KEY_RANGEDOWN``: Represents the RANGE down key.
        - ``.KEY_RANGEUP``: Represents the RANGE up key.
        - ``.KEY_RECALL``: Represents the RECALL key.
        - ``.KEY_RELA``: Represents the REL key.
          For two-channel products, this is the REL key for Channel A.
        - ``.KEY_RIGHT``: Represents the right CURSOR key.
        - ``.KEY_RUN``: Represents the RUN key.
        - ``.KEY_SPEEDA``: Represents the SPEED key.
          For two-channel products, this is the SPEED key for Channel A.
        - ``.KEY_SRCA``: Represents the SRC (Source) key.
          For two-channel products, this is the SRC key for Channel A.
        - ``.KEY_STORE``: Represents the STORE key.
        - ``.KEY_TRIG``: Represents the TRIG key.
        - ``.LIMIT_IV``: Display the primary limit value.
        - ``.LIMIT_P``: Display the power limit value.
        - ``.LOCK``: Unlock the EXIT (LOCAL) key.
        - ``.MEASURE_DCAMPS``: Display the current measurement function.
        - ``.MEASURE_DCVOLTS``: Display the voltage measurement function.
        - ``.MEASURE_OHMS``: Display the resistance measurement function.
        - ``.MEASURE_WATTS``: Display the power measurement function.
        - ``.SAVE``: Save code to nonvolatile memory.
        - ``.SMUA``: Displays source-measure and compliance screen for SMU A.
        - ``.UNLOCK``: Lock out the EXIT (LOCAL) key.
        - ``.USER``: Displays the user screen.
        - ``.WHEEL_ENTER``: Represents pressing the navigation wheel.
        - ``.WHEEL_LEFT``: Represents turning the Navigation wheel left.
        - ``.WHEEL_RIGHT``: Represents turning the Navigation wheel right.

    Properties and methods:
        - ``.getlastkey()``: The ``display.getlastkey()`` function.
        - ``.screen``: The ``display.screen`` attribute.
        - ``.sendkey()``: The ``display.sendkey()`` function.
        - ``.smu``: The ``display.smuX`` command tree.
        - ``.trigger``: The ``display.trigger`` command tree.
        - ``.waitkey()``: The ``display.waitkey()`` function.
    """

    ANNUNCIATOR_4_WIRE = "display.ANNUNCIATOR_4_WIRE"
    """str: 4W indicator on."""
    ANNUNCIATOR_ARM = "display.ANNUNCIATOR_ARM"
    """str: ARM indicator on."""
    ANNUNCIATOR_AUTO = "display.ANNUNCIATOR_AUTO"
    """str: AUTO indicator on."""
    ANNUNCIATOR_EDIT = "display.ANNUNCIATOR_EDIT"
    """str: EDIT indicator on."""
    ANNUNCIATOR_ERROR = "display.ANNUNCIATOR_ERROR"
    """str: ERR indicator on."""
    ANNUNCIATOR_FILTER = "display.ANNUNCIATOR_FILTER"
    """str: FILT indicator on."""
    ANNUNCIATOR_LISTEN = "display.ANNUNCIATOR_LISTEN"
    """str: LSTN indicator on."""
    ANNUNCIATOR_MATH = "display.ANNUNCIATOR_MATH"
    """str: MATH indicator on."""
    ANNUNCIATOR_REAR = "display.ANNUNCIATOR_REAR"
    """str: REAR indicator on."""
    ANNUNCIATOR_REL = "display.ANNUNCIATOR_REL"
    """str: REL indicator on."""
    ANNUNCIATOR_REMOTE = "display.ANNUNCIATOR_REMOTE"
    """str: REM indicator on."""
    ANNUNCIATOR_SAMPLE = "display.ANNUNCIATOR_SAMPLE"
    """str: SMPL indicator on."""
    ANNUNCIATOR_SRQ = "display.ANNUNCIATOR_SRQ"
    """str: SRQ indicator on."""
    ANNUNCIATOR_STAR = "display.ANNUNCIATOR_STAR"
    """str: * (asterisk) indicator on."""
    ANNUNCIATOR_TALK = "display.ANNUNCIATOR_TALK"
    """str: TALK indicator on."""
    ANNUNCIATOR_TRIGGER = "display.ANNUNCIATOR_TRIGGER"
    """str: TRIG indicator on."""
    DIGITS_4_5 = "display.DIGITS_4_5"
    """str: Set the front-panel display resolution to 4.5 digits."""
    DIGITS_5_5 = "display.DIGITS_5_5"
    """str: Set the front-panel display resolution to 5.5 digits."""
    DIGITS_6_5 = "display.DIGITS_6_5"
    """str: Set the front-panel display resolution to 6.5 digits."""
    DISABLE = "display.DISABLE"
    """str: Disable the front-panel keys for numeric entry when entering a value."""
    DONT_SAVE = "display.DONT_SAVE"
    """str: Do not save code to nonvolatile memory."""
    ENABLE = "display.ENABLE"
    """str: Enable the front-panel keys for numeric entry when entering a value."""
    KEY_AUTO = "display.KEY_AUTO"
    """str: Represents the range AUTO key."""
    KEY_CONFIG = "display.KEY_CONFIG"
    """str: Represents the CONFIG key."""
    KEY_DIGITSA = "display.KEY_DIGITSA"
    """str: Represents the DIGITS (display resolution) key.
For two-channel products, this is the DIGITS key for Channel A."""
    KEY_DISPLAY = "display.KEY_DISPLAY"
    """str: Represents the DISPLAY key."""
    KEY_ENTER = "display.KEY_ENTER"
    """str: Represents the ENTER key."""
    KEY_EXIT = "display.KEY_EXIT"
    """str: Represents the EXIT key."""
    KEY_FILTERA = "display.KEY_FILTERA"
    """str: Represents the FILTER key.
For two-channel products, this is the FILTER key for Channel A."""
    KEY_LEFT = "display.KEY_LEFT"
    """str: Represents the left CURSOR key."""
    KEY_LIMITA = "display.KEY_LIMITA"
    """str: Represents the LIMIT key.
For two-channel products, this is the LIMIT key for Channel A."""
    KEY_LOAD = "display.KEY_LOAD"
    """str: Represents the LOAD (load test) key."""
    KEY_MEASA = "display.KEY_MEASA"
    """str: Represents the MEAS (Measure) key.
For two-channel products, this is the MEAS key for Channel A."""
    KEY_MENU = "display.KEY_MENU"
    """str: Represents the MENU key."""
    KEY_MODEA = "display.KEY_MODEA"
    """str: Represents the MODE (meter mode) key
For two-channel products, this is the MODE key for Channel A."""
    KEY_NONE = "display.KEY_NONE"
    """str: No key was pressed."""
    KEY_OUTPUTA = "display.KEY_OUTPUTA"
    """str: Represents the OUTPUT ON/OFF key.
For two-channel products, this is the OUTPUT ON/OFF key for Channel A."""
    KEY_RANGEDOWN = "display.KEY_RANGEDOWN"
    """str: Represents the RANGE down key."""
    KEY_RANGEUP = "display.KEY_RANGEUP"
    """str: Represents the RANGE up key."""
    KEY_RECALL = "display.KEY_RECALL"
    """str: Represents the RECALL key."""
    KEY_RELA = "display.KEY_RELA"
    """str: Represents the REL key.
For two-channel products, this is the REL key for Channel A."""
    KEY_RIGHT = "display.KEY_RIGHT"
    """str: Represents the right CURSOR key."""
    KEY_RUN = "display.KEY_RUN"
    """str: Represents the RUN key."""
    KEY_SPEEDA = "display.KEY_SPEEDA"
    """str: Represents the SPEED key.
For two-channel products, this is the SPEED key for Channel A."""
    KEY_SRCA = "display.KEY_SRCA"
    """str: Represents the SRC (Source) key.
For two-channel products, this is the SRC key for Channel A."""
    KEY_STORE = "display.KEY_STORE"
    """str: Represents the STORE key."""
    KEY_TRIG = "display.KEY_TRIG"
    """str: Represents the TRIG key."""
    LIMIT_IV = "display.LIMIT_IV"
    """str: Display the primary limit value."""
    LIMIT_P = "display.LIMIT_P"
    """str: Display the power limit value."""
    LOCK = "display.LOCK"
    """str: Unlock the EXIT (LOCAL) key."""
    MEASURE_DCAMPS = "display.MEASURE_DCAMPS"
    """str: Display the current measurement function."""
    MEASURE_DCVOLTS = "display.MEASURE_DCVOLTS"
    """str: Display the voltage measurement function."""
    MEASURE_OHMS = "display.MEASURE_OHMS"
    """str: Display the resistance measurement function."""
    MEASURE_WATTS = "display.MEASURE_WATTS"
    """str: Display the power measurement function."""
    SAVE = "display.SAVE"
    """str: Save code to nonvolatile memory."""
    SMUA = "display.SMUA"
    """str: Displays source-measure and compliance screen for SMU A."""
    UNLOCK = "display.UNLOCK"
    """str: Lock out the EXIT (LOCAL) key."""
    USER = "display.USER"
    """str: Displays the user screen."""
    WHEEL_ENTER = "display.WHEEL_ENTER"
    """str: Represents pressing the navigation wheel."""
    WHEEL_LEFT = "display.WHEEL_LEFT"
    """str: Represents turning the Navigation wheel left."""
    WHEEL_RIGHT = "display.WHEEL_RIGHT"
    """str: Represents turning the Navigation wheel right."""

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "display") -> None:
        super().__init__(device, cmd_syntax)
        self._smu: Dict[str, DisplaySmuxItem] = DefaultDictPassKeyToFactory(
            lambda x: DisplaySmuxItem(device, f"{self._cmd_syntax}.smu{x}")
        )
        self._trigger = DisplayTrigger(device, f"{self._cmd_syntax}.trigger")

    @property
    def screen(self) -> str:
        """Access the ``display.screen`` attribute.

        Description:
            - This attribute contains the selected display screen.

        Usage:
            - Accessing this property will send the ``print(display.screen)`` query.
            - Setting this property to a value will send the ``display.screen = value`` command.

        TSP Syntax:
            ```
            - display.screen = value
            - print(display.screen)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".screen"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.screen)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.screen`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @screen.setter
    def screen(self, value: Union[str, float]) -> None:
        """Access the ``display.screen`` attribute.

        Description:
            - This attribute contains the selected display screen.

        Usage:
            - Accessing this property will send the ``print(display.screen)`` query.
            - Setting this property to a value will send the ``display.screen = value`` command.

        TSP Syntax:
            ```
            - display.screen = value
            - print(display.screen)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".screen", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.screen = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.screen`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def smu(self) -> Dict[str, DisplaySmuxItem]:
        """Return the ``display.smuX`` command tree.

        Sub-properties and sub-methods:
            - ``.digits``: The ``display.smuX.digits`` attribute.
            - ``.limit``: The ``display.smuX.limit`` command tree.
            - ``.measure``: The ``display.smuX.measure`` command tree.
        """
        return self._smu

    @property
    def trigger(self) -> DisplayTrigger:
        """Return the ``display.trigger`` command tree.

        Constants:
            - ``.EVENT_ID``: Selects the event that causes a trigger to be asserted on the digital
              output line when the TRIG key on the front panel is pressed.
        """
        return self._trigger

    def getlastkey(self) -> str:
        """Run the ``display.getlastkey()`` function.

        Description:
            - This function retrieves the key code for the last pressed key.

        TSP Syntax:
            ```
            - display.getlastkey()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getlastkey())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.getlastkey()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def sendkey(self, key_code: str) -> None:
        """Run the ``display.sendkey()`` function.

        Description:
            - This function sends a code that simulates the action of a front-panel control.

        TSP Syntax:
            ```
            - display.sendkey()
            ```

        Args:
            key_code: A parameter that specifies the key press to simulate; see Details for more
                information.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.sendkey({key_code})"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.sendkey()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def waitkey(self) -> str:
        """Run the ``display.waitkey()`` function.

        Description:
            - This function captures the key code value for the next front-panel action.

        TSP Syntax:
            ```
            - display.waitkey()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.waitkey())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.waitkey()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The wfmoutpre commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - WFMOutpre:BIT_Nr <NR1>
    - WFMOutpre:BIT_Nr?
    - WFMOutpre:BN_Fmt {RI|RP}
    - WFMOutpre:BN_Fmt?
    - WFMOutpre:BYT_Nr <NR1>
    - WFMOutpre:BYT_Nr?
    - WFMOutpre:BYT_Or {LSB|MSB}
    - WFMOutpre:BYT_Or?
    - WFMOutpre:COMPosition {COMPOSITE_YT|COMPOSITE_ENV|SINGULAR_YT}
    - WFMOutpre:COMPosition?
    - WFMOutpre:ENCdg {ASCii|BINary}
    - WFMOutpre:ENCdg?
    - WFMOutpre:FILTERFreq?
    - WFMOutpre:FRACTional?
    - WFMOutpre:NR_Pt?
    - WFMOutpre:PT_Fmt?
    - WFMOutpre:PT_ORder?
    - WFMOutpre:PT_Off?
    - WFMOutpre:RECOrdlength?
    - WFMOutpre:WFId?
    - WFMOutpre:XINcr?
    - WFMOutpre:XUNit?
    - WFMOutpre:XZEro?
    - WFMOutpre:YMUlt?
    - WFMOutpre:YOFf?
    - WFMOutpre:YUNit?
    - WFMOutpre:YZEro?
    - WFMOutpre?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class WfmoutpreYzero(SCPICmdRead):
    """The ``WFMOutpre:YZEro`` command.

    Description:
        - This query-only command returns the combined vertical position and offset for the source
          waveform specified by ``DATA:SOURCE``. This represents a departure from previous
          instruments where the ``:YZEro`` value represented the vertical position in vertical units
          and the ``:YOFf`` value represented the vertical offset in digitizing levels. For this
          instrument family, the value of ``:YOFf`` is always 0.0. An error is reported if the
          ``DATA:SOURCE`` waveform does not exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:YZEro?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YZEro?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:YZEro?
        ```
    """


class WfmoutpreYunit(SCPICmdRead):
    """The ``WFMOutpre:YUNit`` command.

    Description:
        - This query-only command returns the vertical units for the waveform specified by the
          ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
          exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:YUNit?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YUNit?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:YUNit?
        ```
    """


class WfmoutpreYoff(SCPICmdRead):
    """The ``WFMOutpre:YOFf`` command.

    Description:
        - This query-only command returns the vertical offset of the source specified by
          ``DATA:SOURCE``. For this instrument family, the value returned is always 0.0 as the
          offset is combined with the ``:YZEro`` value. An error is reported if the ``DATA:SOURCE``
          waveform does not exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:YOFf?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YOFf?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:YOFf?
        ```
    """


class WfmoutpreYmult(SCPICmdRead):
    """The ``WFMOutpre:YMUlt`` command.

    Description:
        - This query-only command returns the vertical scale factor per digitizing level in units
          specified by ``WFMOutpre:YUNit`` for the waveform specified by the ``DATA:SOURCE``
          command. For those formats in which ``WFMOUTPRE:BYT_NR`` is important (all non-floating
          point formats), ``WFMOutpre:YMUlt?`` must take the location of the binary point implied by
          ``BYT_NR`` into consideration. An error is reported if the ``DATa:SOUrce`` waveform does
          not exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:YMUlt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:YMUlt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:YMUlt?
        ```
    """


class WfmoutpreXzero(SCPICmdRead):
    """The ``WFMOutpre:XZEro`` command.

    Description:
        - This query-only command returns the sub-sample time between the trigger sample (designated
          by ``PT_OFF``) and the occurrence of the actual trigger for the waveform specified by the
          ``DATA:SOURCE`` command. This value is in units of ``WFMOutpre:XUNit``. An error is
          reported if the ``DATa:SOUrce`` waveform does not exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:XZEro?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XZEro?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:XZEro?
        ```
    """


class WfmoutpreXunit(SCPICmdRead):
    """The ``WFMOutpre:XUNit`` command.

    Description:
        - This query-only command returns the horizontal units for the waveform specified by the
          ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
          exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:XUNit?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XUNit?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:XUNit?
        ```
    """


class WfmoutpreXincr(SCPICmdRead):
    """The ``WFMOutpre:XINcr`` command.

    Description:
        - This query-only command returns the horizontal point spacing in units of
          ``WFMOutpre:XUNit`` for the waveform specified by the ``DATA:SOURCE`` command. This value
          corresponds to the sampling interval. An error is reported if the ``DATa:SOUrce`` waveform
          does not exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:XINcr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:XINcr?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:XINcr?
        ```
    """


class WfmoutpreWfid(SCPICmdRead):
    """The ``WFMOutpre:WFId`` command.

    Description:
        - This query-only command returns a string describing several aspects of the acquisition
          parameters for the waveform specified by the ``DATA:SOURCE`` command. An error is reported
          if the ``DATa:SOUrce`` waveform does not exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:WFId?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:WFId?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:WFId?
        ```
    """


class WfmoutpreRecordlength(SCPICmdRead):
    """The ``WFMOutpre:RECOrdlength`` command.

    Description:
        - Returns the record length for the source waveform as specified by the ``DATA:SOURCE``
          command.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:RECOrdlength?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:RECOrdlength?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:RECOrdlength?
        ```
    """


class WfmoutprePtOff(SCPICmdRead):
    """The ``WFMOutpre:PT_Off`` command.

    Description:
        - This query-only command returns the trigger point relative to ``DATA:START`` for the
          waveform specified by the ``DATA:SOURCE`` command.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_Off?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Off?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:PT_Off?
        ```
    """


class WfmoutprePtOrder(SCPICmdRead):
    """The ``WFMOutpre:PT_ORder`` command.

    Description:
        - This query-only command specifies whether the source waveform is Fast Acquisition. A Fast
          Acquisition waveform is stored as a 500 (vertical) by 1000 (horizontal) point bitmap. Each
          point represents display intensity for that screen location. Only CURVe? query functions
          are allowed on Fast Acquisition waveforms. When the ``WFMOutpre:PT_ORder?`` query returns
          Row, this indicates that the source is a Fast Acquisition waveform (and that each of 1000
          possible horizontal columns being transmitted contains 500 vertical points. Note: 500
          might vary based on how many channels enabled from same FastAcq group.) and waveform
          points are transmitted in the following order: left to right; then top to bottom. When the
          ``WFMOutpre:PT_ORder?`` query returns Linear, this indicates that the source is not a Fast
          Acquisition waveform (and that each horizontal column being sent contains only one
          vertical point). Note that waveform points are transmitted in the following order: top to
          bottom, then left to right.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_ORder?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_ORder?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:PT_ORder?
        ```
    """


class WfmoutprePtFmt(SCPICmdRead):
    """The ``WFMOutpre:PT_Fmt`` command.

    Description:
        - This query-only command returns the point format for the waveform specified by the
          ``DATA:SOURCE`` command. The format specifies a set of equations describing how the scale
          factors in the preamble are used to give meaning to the CURVE data points. An error is
          reported if the ``DATa:SOUrce`` waveform does not exist.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:PT_Fmt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Fmt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:PT_Fmt?
        ```
    """


class WfmoutpreNrPt(SCPICmdRead):
    """The ``WFMOutpre:NR_Pt`` command.

    Description:
        - This query-only command returns the number of points for the ``DATA:SOURCE`` waveform that
          will be transmitted in response to a CURVE? query.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:NR_Pt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:NR_Pt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:NR_Pt?
        ```
    """


class WfmoutpreFractional(SCPICmdRead):
    """The ``WFMOutpre:FRACTional`` command.

    Description:
        - The set form of this command is ignored. The query form always returns a 0, if the
          waveform specified by ``DATA:SOUrce`` is on or displayed. If the waveform is not
          displayed, the query form generates an error and returns event code 2244. This command is
          for compatibility with other Tektronix oscilloscopes.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:FRACTional?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:FRACTional?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:FRACTional?
        ```
    """


class WfmoutpreFilterfreq(SCPICmdRead):
    """The ``WFMOutpre:FILTERFreq`` command.

    Description:
        - Returns the FilterVu low pass filter frequency applied to the LRL waveform record of the
          source waveform specified by ``DATA:SOURCE``. If the ``DATA:RESOLUTION`` is set to FULL,
          then this is the filter frequency applied to the full resolution (LRL) waveform. If the
          ``DATA:RESOLUTION`` is set to REDUced, then this is the filter frequency applied to the
          reduced resolution (thumbnail) waveform.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:FILTERFreq?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:FILTERFreq?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre:FILTERFreq?
        ```
    """


class WfmoutpreEncdg(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:ENCdg`` command.

    Description:
        - This command sets or queries the type of encoding for outgoing waveforms.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:ENCdg?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:ENCdg?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:ENCdg value`` command.

    SCPI Syntax:
        ```
        - WFMOutpre:ENCdg {ASCii|BINary}
        - WFMOutpre:ENCdg?
        ```

    Info:
        - ``ASCii`` specifies that the outgoing data is to be in ASCII format. Waveforms internally
          stored as integers will be sent as <NR1> numbers, while those stored as floating point
          will be sent as <NR3> numbers.
        - ``BINary`` specifies that outgoing data is to be in a binary format whose further
          specification is determined by ``WFMOUTPRE:BYT_NR``, ``WFMOUTPRE:BIT_NR``,
          ``WFMOUTPRE:BN_FMT`` and ``WFMOUTPRE:BYT_OR``.
    """


class WfmoutpreComposition(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:COMPosition`` command.

    Description:
        - Controls or indicates the type of data used by the CURVE? query.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:COMPosition?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:COMPosition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:COMPosition value`` command.

    SCPI Syntax:
        ```
        - WFMOutpre:COMPosition {COMPOSITE_YT|COMPOSITE_ENV|SINGULAR_YT}
        - WFMOutpre:COMPosition?
        ```

    Info:
        - ``COMPOSITE_YT`` uses the sample part of the composite waveform.
        - ``COMPOSITE_ENV`` uses the peak-detect part of the composite waveform.
        - ``SINGULAR_YT`` uses the sample part of the singular waveform.
    """


class WfmoutpreBytOr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BYT_Or`` command.

    Description:
        - This command sets or queries which byte of binary waveform data is transmitted first,
          during a waveform data transfer, when data points require more than one byte. This
          specification only has meaning when ``WFMOUTPRE:ENCDG`` is set to BIN.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Or?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Or?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Or value`` command.

    SCPI Syntax:
        ```
        - WFMOutpre:BYT_Or {LSB|MSB}
        - WFMOutpre:BYT_Or?
        ```

    Info:
        - ``LSB`` specifies that the least significant byte will be transmitted first.
        - ``MSB`` specifies that the most significant byte will be transmitted first.
    """


class WfmoutpreBytNr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BYT_Nr`` command.

    Description:
        - This command sets or queries the binary field data width (bytes per point) for the
          waveform specified by the ``DATA:SOURCE`` command. Note that values will be constrained
          according to the underlying waveform data. This specification is only meaningful when
          ``WFMOUTPRE:ENCDG`` is set to BIN, and ``WFMOUTPRE:BN_FMT`` is set to either RI or RP.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Nr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Nr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Nr value`` command.

    SCPI Syntax:
        ```
        - WFMOutpre:BYT_Nr <NR1>
        - WFMOutpre:BYT_Nr?
        ```

    Info:
        - ``<NR1>`` is the number of bytes per data point and can be 1, 2 or 8. A value of 1 or 2
          bytes per waveform point indicates channel data; 8 bytes per waveform point indicate pixel
          map (fast acquisition) data.
    """


class WfmoutpreBnFmt(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BN_Fmt`` command.

    Description:
        - Sets or returns the format of binary data for outgoing waveforms specified by the
          ``DATA:SOURCE`` command. Changing the value of ``WFMOUTPRE:BN_FMT`` also changes the value
          of ``DATA:ENCDG``.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:BN_Fmt?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BN_Fmt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BN_Fmt value`` command.

    SCPI Syntax:
        ```
        - WFMOutpre:BN_Fmt {RI|RP}
        - WFMOutpre:BN_Fmt?
        ```

    Info:
        - ``RI`` specifies signed integer data point representation.
        - ``RP`` specifies positive integer data point representation.
    """


class WfmoutpreBitNr(SCPICmdWrite, SCPICmdRead):
    """The ``WFMOutpre:BIT_Nr`` command.

    Description:
        - This command sets and queries the number of bits per waveform point that outgoing
          waveforms contain, as specified by the ``DATA:SOURCE`` command. Note that values will be
          constrained according to the underlying waveform data. This specification is only
          meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN and ``WFMOUTPRE:BN_FMT`` is set to
          either RI or RP.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre:BIT_Nr?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre:BIT_Nr?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``WFMOutpre:BIT_Nr value`` command.

    SCPI Syntax:
        ```
        - WFMOutpre:BIT_Nr <NR1>
        - WFMOutpre:BIT_Nr?
        ```

    Info:
        - ``<NR1>`` number of bits per data point can be 8 or 16.
    """


#  pylint: disable=too-many-instance-attributes,too-many-public-methods
class Wfmoutpre(SCPICmdRead):
    """The ``WFMOutpre`` command.

    Description:
        - This query-only command queries the waveform formatting data for the waveform specified by
          the ``DATA:SOURCE`` command. The preamble components are considered to be of two types;
          formatting and interpretation. The formatting components are: ENCdg, ``BN_Fmt``,
          ``BYT_Or``, ``BYT_Nr``, ``BIT_Nr``. The interpretation components are derived from the
          ``DATa:SOUrce`` specified waveform.

    Usage:
        - Using the ``.query()`` method will send the ``WFMOutpre?`` query.
        - Using the ``.verify(value)`` method will send the ``WFMOutpre?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WFMOutpre?
        ```

    Properties:
        - ``.bit_nr``: The ``WFMOutpre:BIT_Nr`` command.
        - ``.bn_fmt``: The ``WFMOutpre:BN_Fmt`` command.
        - ``.byt_nr``: The ``WFMOutpre:BYT_Nr`` command.
        - ``.byt_or``: The ``WFMOutpre:BYT_Or`` command.
        - ``.composition``: The ``WFMOutpre:COMPosition`` command.
        - ``.encdg``: The ``WFMOutpre:ENCdg`` command.
        - ``.filterfreq``: The ``WFMOutpre:FILTERFreq`` command.
        - ``.fractional``: The ``WFMOutpre:FRACTional`` command.
        - ``.nr_pt``: The ``WFMOutpre:NR_Pt`` command.
        - ``.pt_fmt``: The ``WFMOutpre:PT_Fmt`` command.
        - ``.pt_order``: The ``WFMOutpre:PT_ORder`` command.
        - ``.pt_off``: The ``WFMOutpre:PT_Off`` command.
        - ``.recordlength``: The ``WFMOutpre:RECOrdlength`` command.
        - ``.wfid``: The ``WFMOutpre:WFId`` command.
        - ``.xincr``: The ``WFMOutpre:XINcr`` command.
        - ``.xunit``: The ``WFMOutpre:XUNit`` command.
        - ``.xzero``: The ``WFMOutpre:XZEro`` command.
        - ``.ymult``: The ``WFMOutpre:YMUlt`` command.
        - ``.yoff``: The ``WFMOutpre:YOFf`` command.
        - ``.yunit``: The ``WFMOutpre:YUNit`` command.
        - ``.yzero``: The ``WFMOutpre:YZEro`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "WFMOutpre") -> None:
        super().__init__(device, cmd_syntax)
        self._bit_nr = WfmoutpreBitNr(device, f"{self._cmd_syntax}:BIT_Nr")
        self._bn_fmt = WfmoutpreBnFmt(device, f"{self._cmd_syntax}:BN_Fmt")
        self._byt_nr = WfmoutpreBytNr(device, f"{self._cmd_syntax}:BYT_Nr")
        self._byt_or = WfmoutpreBytOr(device, f"{self._cmd_syntax}:BYT_Or")
        self._composition = WfmoutpreComposition(device, f"{self._cmd_syntax}:COMPosition")
        self._encdg = WfmoutpreEncdg(device, f"{self._cmd_syntax}:ENCdg")
        self._filterfreq = WfmoutpreFilterfreq(device, f"{self._cmd_syntax}:FILTERFreq")
        self._fractional = WfmoutpreFractional(device, f"{self._cmd_syntax}:FRACTional")
        self._nr_pt = WfmoutpreNrPt(device, f"{self._cmd_syntax}:NR_Pt")
        self._pt_fmt = WfmoutprePtFmt(device, f"{self._cmd_syntax}:PT_Fmt")
        self._pt_order = WfmoutprePtOrder(device, f"{self._cmd_syntax}:PT_ORder")
        self._pt_off = WfmoutprePtOff(device, f"{self._cmd_syntax}:PT_Off")
        self._recordlength = WfmoutpreRecordlength(device, f"{self._cmd_syntax}:RECOrdlength")
        self._wfid = WfmoutpreWfid(device, f"{self._cmd_syntax}:WFId")
        self._xincr = WfmoutpreXincr(device, f"{self._cmd_syntax}:XINcr")
        self._xunit = WfmoutpreXunit(device, f"{self._cmd_syntax}:XUNit")
        self._xzero = WfmoutpreXzero(device, f"{self._cmd_syntax}:XZEro")
        self._ymult = WfmoutpreYmult(device, f"{self._cmd_syntax}:YMUlt")
        self._yoff = WfmoutpreYoff(device, f"{self._cmd_syntax}:YOFf")
        self._yunit = WfmoutpreYunit(device, f"{self._cmd_syntax}:YUNit")
        self._yzero = WfmoutpreYzero(device, f"{self._cmd_syntax}:YZEro")

    @property
    def bit_nr(self) -> WfmoutpreBitNr:
        """Return the ``WFMOutpre:BIT_Nr`` command.

        Description:
            - This command sets and queries the number of bits per waveform point that outgoing
              waveforms contain, as specified by the ``DATA:SOURCE`` command. Note that values will
              be constrained according to the underlying waveform data. This specification is only
              meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN and ``WFMOUTPRE:BN_FMT`` is set to
              either RI or RP.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:BIT_Nr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BIT_Nr?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BIT_Nr value`` command.

        SCPI Syntax:
            ```
            - WFMOutpre:BIT_Nr <NR1>
            - WFMOutpre:BIT_Nr?
            ```

        Info:
            - ``<NR1>`` number of bits per data point can be 8 or 16.
        """
        return self._bit_nr

    @property
    def bn_fmt(self) -> WfmoutpreBnFmt:
        """Return the ``WFMOutpre:BN_Fmt`` command.

        Description:
            - Sets or returns the format of binary data for outgoing waveforms specified by the
              ``DATA:SOURCE`` command. Changing the value of ``WFMOUTPRE:BN_FMT`` also changes the
              value of ``DATA:ENCDG``.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:BN_Fmt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BN_Fmt?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BN_Fmt value`` command.

        SCPI Syntax:
            ```
            - WFMOutpre:BN_Fmt {RI|RP}
            - WFMOutpre:BN_Fmt?
            ```

        Info:
            - ``RI`` specifies signed integer data point representation.
            - ``RP`` specifies positive integer data point representation.
        """
        return self._bn_fmt

    @property
    def byt_nr(self) -> WfmoutpreBytNr:
        """Return the ``WFMOutpre:BYT_Nr`` command.

        Description:
            - This command sets or queries the binary field data width (bytes per point) for the
              waveform specified by the ``DATA:SOURCE`` command. Note that values will be
              constrained according to the underlying waveform data. This specification is only
              meaningful when ``WFMOUTPRE:ENCDG`` is set to BIN, and ``WFMOUTPRE:BN_FMT`` is set to
              either RI or RP.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Nr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Nr?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Nr value`` command.

        SCPI Syntax:
            ```
            - WFMOutpre:BYT_Nr <NR1>
            - WFMOutpre:BYT_Nr?
            ```

        Info:
            - ``<NR1>`` is the number of bytes per data point and can be 1, 2 or 8. A value of 1 or
              2 bytes per waveform point indicates channel data; 8 bytes per waveform point indicate
              pixel map (fast acquisition) data.
        """
        return self._byt_nr

    @property
    def byt_or(self) -> WfmoutpreBytOr:
        """Return the ``WFMOutpre:BYT_Or`` command.

        Description:
            - This command sets or queries which byte of binary waveform data is transmitted first,
              during a waveform data transfer, when data points require more than one byte. This
              specification only has meaning when ``WFMOUTPRE:ENCDG`` is set to BIN.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:BYT_Or?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:BYT_Or?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:BYT_Or value`` command.

        SCPI Syntax:
            ```
            - WFMOutpre:BYT_Or {LSB|MSB}
            - WFMOutpre:BYT_Or?
            ```

        Info:
            - ``LSB`` specifies that the least significant byte will be transmitted first.
            - ``MSB`` specifies that the most significant byte will be transmitted first.
        """
        return self._byt_or

    @property
    def composition(self) -> WfmoutpreComposition:
        """Return the ``WFMOutpre:COMPosition`` command.

        Description:
            - Controls or indicates the type of data used by the CURVE? query.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:COMPosition?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:COMPosition?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:COMPosition value``
              command.

        SCPI Syntax:
            ```
            - WFMOutpre:COMPosition {COMPOSITE_YT|COMPOSITE_ENV|SINGULAR_YT}
            - WFMOutpre:COMPosition?
            ```

        Info:
            - ``COMPOSITE_YT`` uses the sample part of the composite waveform.
            - ``COMPOSITE_ENV`` uses the peak-detect part of the composite waveform.
            - ``SINGULAR_YT`` uses the sample part of the singular waveform.
        """
        return self._composition

    @property
    def encdg(self) -> WfmoutpreEncdg:
        """Return the ``WFMOutpre:ENCdg`` command.

        Description:
            - This command sets or queries the type of encoding for outgoing waveforms.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:ENCdg?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:ENCdg?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``WFMOutpre:ENCdg value`` command.

        SCPI Syntax:
            ```
            - WFMOutpre:ENCdg {ASCii|BINary}
            - WFMOutpre:ENCdg?
            ```

        Info:
            - ``ASCii`` specifies that the outgoing data is to be in ASCII format. Waveforms
              internally stored as integers will be sent as <NR1> numbers, while those stored as
              floating point will be sent as <NR3> numbers.
            - ``BINary`` specifies that outgoing data is to be in a binary format whose further
              specification is determined by ``WFMOUTPRE:BYT_NR``, ``WFMOUTPRE:BIT_NR``,
              ``WFMOUTPRE:BN_FMT`` and ``WFMOUTPRE:BYT_OR``.
        """
        return self._encdg

    @property
    def filterfreq(self) -> WfmoutpreFilterfreq:
        """Return the ``WFMOutpre:FILTERFreq`` command.

        Description:
            - Returns the FilterVu low pass filter frequency applied to the LRL waveform record of
              the source waveform specified by ``DATA:SOURCE``. If the ``DATA:RESOLUTION`` is set to
              FULL, then this is the filter frequency applied to the full resolution (LRL) waveform.
              If the ``DATA:RESOLUTION`` is set to REDUced, then this is the filter frequency
              applied to the reduced resolution (thumbnail) waveform.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:FILTERFreq?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:FILTERFreq?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:FILTERFreq?
            ```
        """
        return self._filterfreq

    @property
    def fractional(self) -> WfmoutpreFractional:
        """Return the ``WFMOutpre:FRACTional`` command.

        Description:
            - The set form of this command is ignored. The query form always returns a 0, if the
              waveform specified by ``DATA:SOUrce`` is on or displayed. If the waveform is not
              displayed, the query form generates an error and returns event code 2244. This command
              is for compatibility with other Tektronix oscilloscopes.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:FRACTional?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:FRACTional?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:FRACTional?
            ```
        """
        return self._fractional

    @property
    def nr_pt(self) -> WfmoutpreNrPt:
        """Return the ``WFMOutpre:NR_Pt`` command.

        Description:
            - This query-only command returns the number of points for the ``DATA:SOURCE`` waveform
              that will be transmitted in response to a CURVE? query.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:NR_Pt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:NR_Pt?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:NR_Pt?
            ```
        """
        return self._nr_pt

    @property
    def pt_fmt(self) -> WfmoutprePtFmt:
        """Return the ``WFMOutpre:PT_Fmt`` command.

        Description:
            - This query-only command returns the point format for the waveform specified by the
              ``DATA:SOURCE`` command. The format specifies a set of equations describing how the
              scale factors in the preamble are used to give meaning to the CURVE data points. An
              error is reported if the ``DATa:SOUrce`` waveform does not exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_Fmt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Fmt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:PT_Fmt?
            ```
        """
        return self._pt_fmt

    @property
    def pt_order(self) -> WfmoutprePtOrder:
        """Return the ``WFMOutpre:PT_ORder`` command.

        Description:
            - This query-only command specifies whether the source waveform is Fast Acquisition. A
              Fast Acquisition waveform is stored as a 500 (vertical) by 1000 (horizontal) point
              bitmap. Each point represents display intensity for that screen location. Only CURVe?
              query functions are allowed on Fast Acquisition waveforms. When the
              ``WFMOutpre:PT_ORder?`` query returns Row, this indicates that the source is a Fast
              Acquisition waveform (and that each of 1000 possible horizontal columns being
              transmitted contains 500 vertical points. Note: 500 might vary based on how many
              channels enabled from same FastAcq group.) and waveform points are transmitted in the
              following order: left to right; then top to bottom. When the ``WFMOutpre:PT_ORder?``
              query returns Linear, this indicates that the source is not a Fast Acquisition
              waveform (and that each horizontal column being sent contains only one vertical
              point). Note that waveform points are transmitted in the following order: top to
              bottom, then left to right.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_ORder?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_ORder?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:PT_ORder?
            ```
        """
        return self._pt_order

    @property
    def pt_off(self) -> WfmoutprePtOff:
        """Return the ``WFMOutpre:PT_Off`` command.

        Description:
            - This query-only command returns the trigger point relative to ``DATA:START`` for the
              waveform specified by the ``DATA:SOURCE`` command.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:PT_Off?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:PT_Off?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:PT_Off?
            ```
        """
        return self._pt_off

    @property
    def recordlength(self) -> WfmoutpreRecordlength:
        """Return the ``WFMOutpre:RECOrdlength`` command.

        Description:
            - Returns the record length for the source waveform as specified by the ``DATA:SOURCE``
              command.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:RECOrdlength?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:RECOrdlength?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:RECOrdlength?
            ```
        """
        return self._recordlength

    @property
    def wfid(self) -> WfmoutpreWfid:
        """Return the ``WFMOutpre:WFId`` command.

        Description:
            - This query-only command returns a string describing several aspects of the acquisition
              parameters for the waveform specified by the ``DATA:SOURCE`` command. An error is
              reported if the ``DATa:SOUrce`` waveform does not exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:WFId?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:WFId?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:WFId?
            ```
        """
        return self._wfid

    @property
    def xincr(self) -> WfmoutpreXincr:
        """Return the ``WFMOutpre:XINcr`` command.

        Description:
            - This query-only command returns the horizontal point spacing in units of
              ``WFMOutpre:XUNit`` for the waveform specified by the ``DATA:SOURCE`` command. This
              value corresponds to the sampling interval. An error is reported if the
              ``DATa:SOUrce`` waveform does not exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:XINcr?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XINcr?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:XINcr?
            ```
        """
        return self._xincr

    @property
    def xunit(self) -> WfmoutpreXunit:
        """Return the ``WFMOutpre:XUNit`` command.

        Description:
            - This query-only command returns the horizontal units for the waveform specified by the
              ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:XUNit?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XUNit?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:XUNit?
            ```
        """
        return self._xunit

    @property
    def xzero(self) -> WfmoutpreXzero:
        """Return the ``WFMOutpre:XZEro`` command.

        Description:
            - This query-only command returns the sub-sample time between the trigger sample
              (designated by ``PT_OFF``) and the occurrence of the actual trigger for the waveform
              specified by the ``DATA:SOURCE`` command. This value is in units of
              ``WFMOutpre:XUNit``. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:XZEro?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:XZEro?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:XZEro?
            ```
        """
        return self._xzero

    @property
    def ymult(self) -> WfmoutpreYmult:
        """Return the ``WFMOutpre:YMUlt`` command.

        Description:
            - This query-only command returns the vertical scale factor per digitizing level in
              units specified by ``WFMOutpre:YUNit`` for the waveform specified by the
              ``DATA:SOURCE`` command. For those formats in which ``WFMOUTPRE:BYT_NR`` is important
              (all non-floating point formats), ``WFMOutpre:YMUlt?`` must take the location of the
              binary point implied by ``BYT_NR`` into consideration. An error is reported if the
              ``DATa:SOUrce`` waveform does not exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:YMUlt?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YMUlt?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:YMUlt?
            ```
        """
        return self._ymult

    @property
    def yoff(self) -> WfmoutpreYoff:
        """Return the ``WFMOutpre:YOFf`` command.

        Description:
            - This query-only command returns the vertical offset of the source specified by
              ``DATA:SOURCE``. For this instrument family, the value returned is always 0.0 as the
              offset is combined with the ``:YZEro`` value. An error is reported if the
              ``DATA:SOURCE`` waveform does not exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:YOFf?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YOFf?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:YOFf?
            ```
        """
        return self._yoff

    @property
    def yunit(self) -> WfmoutpreYunit:
        """Return the ``WFMOutpre:YUNit`` command.

        Description:
            - This query-only command returns the vertical units for the waveform specified by the
              ``DATA:SOURCE`` command. An error is reported if the ``DATa:SOUrce`` waveform does not
              exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:YUNit?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YUNit?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:YUNit?
            ```
        """
        return self._yunit

    @property
    def yzero(self) -> WfmoutpreYzero:
        """Return the ``WFMOutpre:YZEro`` command.

        Description:
            - This query-only command returns the combined vertical position and offset for the
              source waveform specified by ``DATA:SOURCE``. This represents a departure from
              previous instruments where the ``:YZEro`` value represented the vertical position in
              vertical units and the ``:YOFf`` value represented the vertical offset in digitizing
              levels. For this instrument family, the value of ``:YOFf`` is always 0.0. An error is
              reported if the ``DATA:SOURCE`` waveform does not exist.

        Usage:
            - Using the ``.query()`` method will send the ``WFMOutpre:YZEro?`` query.
            - Using the ``.verify(value)`` method will send the ``WFMOutpre:YZEro?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - WFMOutpre:YZEro?
            ```
        """
        return self._yzero



"""Base MF 5xxx device driver module."""

from __future__ import annotations

import inspect
import logging

from abc import ABC
from typing import List, Tuple, TYPE_CHECKING, Union

from tm_devices.driver_mixins.device_control.tsp_control import TSPControl
from tm_devices.driver_mixins.shared_implementations import CommonTSPErrorCheckMixin
from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.mainframes.mainframe import Mainframe

# noinspection PyPep8Naming
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813

if TYPE_CHECKING:
    from tm_devices.commands import MPSU50_2STCommands
    from tm_devices.commands.gen_3skc3w_mp.slot import SlotItem


_POWER_SUPPLY_UNIT_MODULES = ["MPSU50-2ST"]

_logger: logging.Logger = logging.getLogger(__name__)


@family_base_class
class MP5xxx(CommonTSPErrorCheckMixin, TSPControl, Mainframe, ABC):
    """Base MP5xxx device driver."""

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_slot_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the slot names."""
        return tuple(f"{x + 1}" for x in range(self.total_slots))

    @cached_property
    def total_slots(self) -> int:
        """Return the total number of slots."""
        return int(self.model[5])

    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names of a module."""
        return tuple(f"{x + 1}" for x in range(self.total_channels))

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels across all modules."""
        total_channels = 0
        for slot in range(1, self.total_slots + 1):
            model_returned = self.query(f"print(slot[{slot}].model)")
            # TODO: Modify the logic to determine channel based on return type
            channels_in_module = int(
                next((char for char in reversed(str(model_returned)) if char.isdigit()), 0)
            )
            total_channels += channels_in_module
        return total_channels

    ################################################################################################
    # Public Methods
    ################################################################################################
    def get_module_commands_psu(self, slot: int) -> Union[MPSU50_2STCommands, SlotItem]:
        """Get a power supply unit (PSU) module commands object from the mainframe.

        Args:
            slot (int): The slot number of the PSU module.

        Returns:
            The PSU commands associated with the specified slot.

        Raises:
            ValueError: If no module is available in the specified slot.
            TypeError: If the module in the slot is not one of the supported PSU modules.
        """
        return self._get_module_commands_api("PSU", _POWER_SUPPLY_UNIT_MODULES, slot)

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _get_module_commands_api(
        self, module_type: str, module_alias: List[str], slot: int
    ) -> SlotItem:
        """Get the module commands object for the specific module_type.

        Args:
            module_type : The type of module being accessed.
            module_alias: List of supported module aliases.
            slot: The slot number of the module.

        Returns:
            The commands API for the module in the given slot.

        Raises:
            ValueError: If the slot number is not between one and the total
                        number of slots (inclusive), if no module is available
                        in the specified slot, or if an invalid module is detected.
            TypeError: If the slot number is not an integer, or
                       if the module in the slot is not supported.
        """
        if not isinstance(slot, int):  # pyright: ignore[reportUnnecessaryIsInstance]
            msg = (
                f"{slot} of type {type(slot).__name__} is not a valid slot number."
                " A valid slot number must be an integer."
            )
            raise TypeError(msg)

        if not 1 <= slot <= self.total_slots:
            msg = (
                f"Slot number {slot} is out of range. It must be between 1 and {self.total_slots}."
            )
            raise ValueError(msg)

        try:
            model_returned = self.query(f"print(slot[{slot}].model)")
            if model_returned.find("-") == -1:
                msg = f"{model_returned} Module in slot {slot} is an invalid module."
                raise ValueError(msg)  # noqa: TRY301

            if model_returned in module_alias:
                _logger.debug("Module '%s' commands are accessible.", model_returned)
                return self.commands.slot[slot]

            supported_modules = ", ".join(module_alias)

            installed_module = self.query(f"print(slot[{slot}].model)")
            msg = (
                f"No supported {module_type} module found in slot {slot}. "
                f"The supported {module_type} modules are: {supported_modules}. "
                f"Currently installed module: {installed_module}."
            )
            raise TypeError(msg)
        except ValueError as error:
            msg = f"Error encountered while accessing module commands object: {error}"
            raise ValueError(msg) from error

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _reboot(self) -> None:
        """Perform the actual rebooting code.

        Raises:
            NotImplementedError: Indicates the current driver has not implemented this method.
        """
        # TODO: implement
        raise NotImplementedError(
            f"``.{inspect.currentframe().f_code.co_name}()``"  # pyright: ignore[reportOptionalMemberAccess]  # noqa: EM102
            f" is not yet implemented for the {self.__class__.__name__} driver"
        )
